@@ -16,6 +16,9 @@ impl UsageAnalyzer {
 
 #[derive(Debug, Default, Clone, Copy)]
 pub(super) struct Ctx {
+    /// See [crate::marks::Marks]
+    pub skip_standalone: bool,
+
     pub var_decl_kind_of_pat: Option<VarDeclKind>,
 
     pub in_var_decl_with_no_side_effect_for_member_access: bool,
