@@ -4,45 +4,50 @@ use self::optimize::optimizer;
 use self::optimize::OptimizerState;
 use crate::analyzer::analyze;
 use crate::analyzer::ProgramData;
+use crate::analyzer::UsageAnalyzer;
 use crate::compress::hoist_decls::decl_hoister;
+use crate::compress::pure::pure_optimizer;
 use crate::debug::dump;
 use crate::debug::invoke;
 use crate::marks::Marks;
 use crate::option::CompressOptions;
 use crate::util::now;
+use crate::util::unit::CompileUnit;
 use crate::util::Optional;
+use crate::MAX_PAR_DEPTH;
 #[cfg(feature = "pretty_assertions")]
 use pretty_assertions::assert_eq;
+use rayon::prelude::*;
 use std::borrow::Cow;
 use std::fmt;
 use std::fmt::Debug;
 use std::fmt::Display;
 use std::fmt::Formatter;
+use std::thread;
 use std::time::Instant;
+use swc_common::chain;
 use swc_common::pass::CompilerPass;
-use swc_common::pass::Repeat;
 use swc_common::pass::Repeated;
-use swc_common::sync::Lrc;
-use swc_common::{chain, SourceMap};
+use swc_common::Globals;
 use swc_ecma_ast::*;
-use swc_ecma_transforms::fixer;
 use swc_ecma_transforms::optimization::simplify::dead_branch_remover;
 use swc_ecma_transforms::optimization::simplify::expr_simplifier;
 use swc_ecma_transforms::pass::JsPass;
-use swc_ecma_transforms_base::ext::MapWithMut;
 use swc_ecma_utils::StmtLike;
 use swc_ecma_visit::as_folder;
 use swc_ecma_visit::noop_visit_mut_type;
-use swc_ecma_visit::FoldWith;
 use swc_ecma_visit::VisitMut;
 use swc_ecma_visit::VisitMutWith;
+use swc_ecma_visit::VisitWith;
 
 mod drop_console;
 mod hoist_decls;
 mod optimize;
+mod pure;
+mod util;
 
 pub(crate) fn compressor<'a>(
-    cm: Lrc<SourceMap>,
+    globals: &'a Globals,
     marks: Marks,
     options: &'a CompressOptions,
 ) -> impl 'a + JsPass {
@@ -51,30 +56,29 @@ pub(crate) fn compressor<'a>(
         visitor: drop_console(),
     };
     let compressor = Compressor {
-        cm,
+        globals,
         marks,
         options,
         changed: false,
         pass: 0,
         data: None,
         optimizer_state: Default::default(),
+        left_parallel_depth: 0,
     };
 
-    chain!(
-        console_remover,
-        Repeat::new(as_folder(compressor)),
-        expr_simplifier()
-    )
+    chain!(console_remover, as_folder(compressor), expr_simplifier())
 }
 
 struct Compressor<'a> {
-    cm: Lrc<SourceMap>,
+    globals: &'a Globals,
     marks: Marks,
     options: &'a CompressOptions,
     changed: bool,
     pass: usize,
     data: Option<ProgramData>,
     optimizer_state: OptimizerState,
+    /// `0` means we should not create more threads.
+    left_parallel_depth: u8,
 }
 
 impl CompilerPass for Compressor<'_> {
@@ -83,18 +87,6 @@ impl CompilerPass for Compressor<'_> {
     }
 }
 
-impl Repeated for Compressor<'_> {
-    fn changed(&self) -> bool {
-        self.changed
-    }
-
-    fn reset(&mut self) {
-        self.changed = false;
-        self.pass += 1;
-        self.data = None;
-    }
-}
-
 impl Compressor<'_> {
     fn handle_stmt_likes<T>(&mut self, stmts: &mut Vec<T>)
     where
@@ -123,14 +115,108 @@ impl Compressor<'_> {
 
         // TODO: drop unused
     }
-}
 
-impl VisitMut for Compressor<'_> {
-    noop_visit_mut_type!();
+    /// Optimize a bundle in a parallel.
+    fn visit_par<N>(&mut self, nodes: &mut Vec<N>)
+    where
+        N: Send + Sync + for<'aa> VisitMutWith<Compressor<'aa>>,
+    {
+        log::debug!("visit_par(left_depth = {})", self.left_parallel_depth);
 
-    fn visit_mut_module(&mut self, n: &mut Module) {
-        debug_assert!(self.data.is_none());
-        self.data = Some(analyze(&*n, self.marks));
+        if self.left_parallel_depth == 0 || cfg!(target_arch = "wasm32") {
+            for node in nodes {
+                let mut v = Compressor {
+                    globals: self.globals,
+                    marks: self.marks,
+                    options: self.options,
+                    changed: false,
+                    pass: self.pass,
+                    data: None,
+                    optimizer_state: Default::default(),
+                    left_parallel_depth: 0,
+                };
+                node.visit_mut_with(&mut v);
+
+                self.changed |= v.changed;
+            }
+        } else {
+            let results = nodes
+                .par_iter_mut()
+                .map(|node| {
+                    swc_common::GLOBALS.set(&self.globals, || {
+                        let mut v = Compressor {
+                            globals: self.globals,
+                            marks: self.marks,
+                            options: self.options,
+                            changed: false,
+                            pass: self.pass,
+                            data: None,
+                            optimizer_state: Default::default(),
+                            left_parallel_depth: self.left_parallel_depth - 1,
+                        };
+                        node.visit_mut_with(&mut v);
+
+                        v.changed
+                    })
+                })
+                .collect::<Vec<_>>();
+
+            for changed in results {
+                self.changed |= changed;
+            }
+        }
+    }
+
+    fn optimize_unit_repeatedly<N>(&mut self, n: &mut N)
+    where
+        N: CompileUnit + VisitWith<UsageAnalyzer> + for<'aa> VisitMutWith<Compressor<'aa>>,
+    {
+        if cfg!(feature = "debug") {
+            log::debug!(
+                "Optimizing a compile unit within `{:?}`",
+                thread::current().name()
+            );
+        }
+
+        {
+            let data = analyze(&*n, Some(self.marks));
+
+            let mut v = decl_hoister(
+                DeclHoisterConfig {
+                    hoist_fns: self.options.hoist_fns,
+                    hoist_vars: self.options.hoist_vars,
+                    top_level: self.options.top_level(),
+                },
+                &data,
+            );
+            n.apply(&mut v);
+            self.changed |= v.changed();
+        }
+
+        loop {
+            self.changed = false;
+            self.optimize_unit(n);
+            self.pass += 1;
+            if !self.changed {
+                break;
+            }
+        }
+
+        // let last_mark = n.remove_mark();
+        // assert!(
+        //     N::is_module() || last_mark == self.marks.standalone,
+        //     "{:?}; last={:?}",
+        //     self.marks,
+        //     last_mark
+        // );
+    }
+
+    /// Optimize a module. `N` can be [Module] or [FnExpr].
+    fn optimize_unit<N>(&mut self, n: &mut N)
+    where
+        N: CompileUnit + VisitWith<UsageAnalyzer> + for<'aa> VisitMutWith<Compressor<'aa>>,
+    {
+        self.data = Some(analyze(&*n, Some(self.marks)));
 
         if self.options.passes != 0 && self.options.passes + 1 <= self.pass {
             let done = dump(&*n);
@@ -144,7 +230,7 @@ impl VisitMut for Compressor<'_> {
         }
 
         let start = if cfg!(feature = "debug") {
-            let start = dump(&n.clone().fold_with(&mut fixer(None)));
+            let start = n.dump();
             log::debug!("===== Start =====\n{}", start);
             start
         } else {
@@ -160,7 +246,7 @@ impl VisitMut for Compressor<'_> {
             let start_time = now();
 
             let mut visitor = expr_simplifier();
-            n.visit_mut_with(&mut visitor);
+            n.apply(&mut visitor);
             self.changed |= visitor.changed();
             if visitor.changed() {
                 log::debug!("compressor: Simplified expressions");
@@ -180,7 +266,7 @@ impl VisitMut for Compressor<'_> {
             }
 
             if cfg!(feature = "debug") && !visitor.changed() {
-                let simplified = dump(&n.clone().fold_with(&mut fixer(None)));
+                let simplified = n.dump();
                 if start != simplified {
                     assert_eq!(
                         DebugUsingDisplay(&start),
@@ -192,6 +278,29 @@ impl VisitMut for Compressor<'_> {
             }
         }
 
+        {
+            log::info!(
+                "compress/pure: Running pure optimizer (pass = {})",
+                self.pass
+            );
+
+            let start_time = now();
+
+            let mut visitor = pure_optimizer(&self.options, self.marks);
+            n.apply(&mut visitor);
+            self.changed |= visitor.changed();
+
+            if let Some(start_time) = start_time {
+                let end_time = Instant::now();
+
+                log::info!(
+                    "compress/pure: Pure optimizer took {:?} (pass = {})",
+                    end_time - start_time,
+                    self.pass
+                );
+            }
+        }
+
         {
             log::info!("compress: Running optimizer (pass = {})", self.pass);
 
@@ -203,13 +312,12 @@ impl VisitMut for Compressor<'_> {
             self.optimizer_state = Default::default();
 
             let mut visitor = optimizer(
-                self.cm.clone(),
                 self.marks,
                 self.options,
                 self.data.as_ref().unwrap(),
                 &mut self.optimizer_state,
             );
-            n.visit_mut_with(&mut visitor);
+            n.apply(&mut visitor);
             self.changed |= visitor.changed();
 
             if let Some(start_time) = start_time {
@@ -235,7 +343,7 @@ impl VisitMut for Compressor<'_> {
             let start_time = now();
 
             let mut v = dead_branch_remover();
-            n.map_with_mut(|n| n.fold_with(&mut v));
+            n.apply(&mut v);
 
             if let Some(start_time) = start_time {
                 let end_time = Instant::now();
@@ -261,26 +369,39 @@ impl VisitMut for Compressor<'_> {
 
             self.changed |= v.changed();
         }
+    }
+}
+
+impl VisitMut for Compressor<'_> {
+    noop_visit_mut_type!();
+
+    fn visit_mut_fn_expr(&mut self, n: &mut FnExpr) {
+        if n.function.span.has_mark(self.marks.standalone) {
+            self.optimize_unit_repeatedly(n);
+            return;
+        }
 
         n.visit_mut_children_with(self);
+    }
+
+    fn visit_mut_module(&mut self, n: &mut Module) {
+        let is_bundle_mode = n.span.has_mark(self.marks.bundle_of_standalones);
+
+        if is_bundle_mode {
+            self.left_parallel_depth = MAX_PAR_DEPTH - 1;
+        } else {
+            self.optimize_unit_repeatedly(n);
+            return;
+        }
+
+        n.visit_mut_children_with(self);
+
+        self.optimize_unit_repeatedly(n);
 
         invoke(&*n);
     }
 
     fn visit_mut_module_items(&mut self, stmts: &mut Vec<ModuleItem>) {
-        {
-            let mut v = decl_hoister(
-                DeclHoisterConfig {
-                    hoist_fns: self.options.hoist_fns,
-                    hoist_vars: self.options.hoist_vars,
-                    top_level: self.options.top_level(),
-                },
-                self.data.as_ref().unwrap(),
-            );
-            stmts.visit_mut_with(&mut v);
-            self.changed |= v.changed();
-        }
-
         self.handle_stmt_likes(stmts);
 
         stmts.retain(|stmt| match stmt {
@@ -298,24 +419,8 @@ impl VisitMut for Compressor<'_> {
         });
     }
 
-    fn visit_mut_script(&mut self, n: &mut Script) {
-        debug_assert!(self.data.is_none());
-        self.data = Some(analyze(&*n, self.marks));
-
-        {
-            let mut v = decl_hoister(
-                DeclHoisterConfig {
-                    hoist_fns: self.options.hoist_fns,
-                    hoist_vars: self.options.hoist_vars,
-                    top_level: self.options.top_level(),
-                },
-                self.data.as_ref().unwrap(),
-            );
-            n.body.visit_mut_with(&mut v);
-            self.changed |= v.changed();
-        }
-
-        n.visit_mut_children_with(self);
+    fn visit_mut_prop_or_spreads(&mut self, nodes: &mut Vec<PropOrSpread>) {
+        self.visit_par(nodes);
     }
 
     fn visit_mut_stmts(&mut self, stmts: &mut Vec<Stmt>) {
