@@ -1,5 +1,5 @@
 use super::Optimizer;
-use crate::compress::optimize::is_pure_undefined;
+use crate::compress::util::is_pure_undefined;
 use crate::debug::dump;
 use crate::util::ExprOptExt;
 use swc_common::Spanned;
@@ -17,19 +17,6 @@ use swc_ecma_visit::VisitWith;
 /// Methods related to the option `if_return`. All methods are noop if
 /// `if_return` is false.
 impl Optimizer<'_> {
-    pub(super) fn drop_undefined_from_return_arg(&mut self, s: &mut ReturnStmt) {
-        match s.arg.as_deref() {
-            Some(e) => {
-                if is_pure_undefined(e) {
-                    self.changed = true;
-                    log::debug!("Dropped `undefined` from `return undefined`");
-                    s.arg.take();
-                }
-            }
-            None => {}
-        }
-    }
-
     /// # Input
     ///
     /// ```js
